if not CompactBars then
	error("CompactBars has to be loaded first")
end

local newList, del, deepDel, deepCopy = CompactBars.sb:GetRecyclingFunctions()
local bossmods = CompactBars:NewModule("bossmods")
local ctab = {
	[1] = {r = 0, g = 0, b = 1, a = 0.25},
	[2] = {r = 1, g = 0, b = 0, a = 0.25},
	[3] = {r = 0, g = 1, b = 0, a = 0.25},
	[4] = {r = 0, g = 0, b = 1, a = 0.25},
	[5] = {r = 1, g = 1, b = 1, a = 0.25},
	[6] = {r = 205/255, g = 127/255, b = 50/255, a = 0.25 },
	[7] = {r = 0, g = 100/255, b = 0, a = 0.25 },
	[8] = {r = 1, g = 20/255, b = 147/255, a = 0.25 },
	[9] = {r = 1, g = 1, b = 0, a = 0.25},
}

local count = 0
local function getColor()
	count = count + 1
	if count > #ctab then count = 1 end
	return ctab[count]
end
function CompactBars:removeColor()
	count = count - 1
end
--self, timer, id, icon, huge, small, color
local function dbmHook(...)
	if CompactBars.db.profile.ebossmods then
		local text
		if not CompactBars.db.profile.bossmodbartext then
			text = id
		else
			text = nil
		end
		local color = getColor()
		if string.find(id, "dummy") then
			CompactBars.hooks[self]["CreateBar"](self, timer, id, icon, huge, small, color)
			return
		end
		CompactBars.sb:CreateBar({id = id, typ = "Bossmods"}, timer, 0, text, icon, CompactBars:GetBarPos(0, "Bossmods"), deepCopy(color), deepCopy(color), CompactBars.db.profile.bartexture, function() CompactBars:removeColor() end)
		CompactBars:updateBarPos()
		if not CompactBars.db.profile.hidebossmod then
			CompactBars.hooks[self]["CreateBar"](self, timer, id, icon, huge, small, color)
		end
	else
		CompactBars.hooks[self]["CreateBar"](self, timer, id, icon, huge, small, color)
	end
end
local function dbmStopHook(self, id)
	if CompactBars.db.profile.ebossmods then
		local bars = CompactBars.sb:GetBars()
		for _, bar in pairs(bars) do
			if bar.data and bar.data.id == id then
				bar.start = nil
			end
		end
	end
end
local function bigwigs(self, id, dur, icon)
	if CompactBars.db.profile.ebossmods then
		local color = getColor()
		local text
		if not CompactBars.db.profile.bossmodbartext then
			text = id
		else
			text = nil
		end
		CompactBars.sb:CreateBar({id = id, typ = "Bossmods"}, dur, 0, text, icon, CompactBars:GetBarPos(0, "Bossmods"), deepCopy(color), deepCopy(color), CompactBars.db.profile.bartexture, function() CompactBars:removeColor() end)
		CompactBars:updateBarPos()
	end
end
local function bigwigs_stop(self, id)
	if CompactBars.db.profile.ebossmods then
		local bars = CompactBars.sb:GetBars()
		for _, bar in pairs(bars) do
			if bar.data and bar.data.id == id then
				bar.start = nil
			end
		end
	end
end

local function bwBarModuleLoaded(name)
	if CompactBars.db.profile.ebossmods and CompactBars.db.profile.hidebossmod then
		if name == "Bars" then
			CompactBars:ScheduleTimer(function() BigWigs:ToggleModuleActive("Bars", false) end, 1)
		end
	end
end
local function bwWipeBars()
	local wpos = CompactBars:GetBarPos(0, "Bossmods")
	local bars = CompactBars.sb:GetBars()
	if not bars or #bars == 0 then return end
	for _, bar in bars do 
		if bar and bar.pos and bar.pos == wpos then
			bar.start = nil
		end
	end
end
local function setupHooks()
	if IsAddOnLoaded("BigWigs") then
		AceLibrary("AceEvent-2.0"):RegisterEvent("BigWigs_StartBar", bigwigs)
		AceLibrary("AceEvent-2.0"):RegisterEvent("BigWigs_StopBar", bigwigs_stop)
		AceLibrary("AceEvent-2.0"):RegisterEvent("BigWigs_ModuleRegistered", bwBarModuleLoaded)
		AceLibrary("AceEvent-2.0"):RegisterEvent("BigWigs_RebootModule", bwWipeBars)
		
	elseif IsAddOnLoaded("DBM-Core") then
		CompactBars:RawHook(DBM.Bars, "CreateBar", function(...) dbmHook(...) end)
		CompactBars:RawHook(DBM.Bars, "CancelBar", function(...) dbmStopHook(...) end)
	end
end

function bossmods:OnInitialize()
	if not CompactBars.db.profile.ebossmods then return end
	setupHooks()
end
